#include "CudaUtilities.h"

#include <stdio.h>



__global__ void CudaI420_To_RGBA(unsigned char *device_rgba , unsigned char *device_I420_Y , unsigned char *device_I420_UV , int width , int  height)
{

	uint32_t rowIndex = blockIdx.y*blockDim.y + threadIdx.y;
	uint32_t colIndex = blockIdx.x*blockDim.x + threadIdx.x;

	if( rowIndex >= height || colIndex >= width )
		return;

	int r, g, b, a;
	uint32_t index = (rowIndex * width) + colIndex;
	uint32_t Y_index, U_index, V_index ;

	
	Y_index = index;
	U_index = (((rowIndex >> 1) * width + ((colIndex >> 1) << 1))) >> 1;
	V_index =  U_index + ((height*width) >> 2);

	r = device_I420_Y[Y_index] + 1.4065 * (device_I420_UV[V_index] - 128);

	g = device_I420_Y[Y_index] - 0.3455 * (device_I420_UV[U_index] - 128) - 0.7169 * (device_I420_UV[V_index] - 128);

	b = device_I420_Y[Y_index] + 1.7790 * (device_I420_UV[U_index] - 128);

	r = r < 0 ? 0 : r;
	g = g < 0 ? 0 : g;
	b = b < 0 ? 0 : b;

	r = r > 255 ? 255 : r;
	g = g > 255 ? 255 : g;
	b = b > 255 ? 255 : b;

	a = 0;

	device_rgba[4 * index] = r;
	device_rgba[4 * index + 1] = g;
	device_rgba[4 * index + 2] = b;
	device_rgba[4 * index + 3] = a;
}


void CudaI420_To_RGBA_Wrapper(unsigned char *cuda_rgba_buffer, unsigned char *cuda_i420_buffer_Y, unsigned char *cuda_i420_buffer_UV, int width, int height, int block_dim_x, int block_dim_y)
{
	dim3 threads(block_dim_x, block_dim_y);
	dim3 blocks((width+block_dim_x-1)/block_dim_x, (height + block_dim_y - 1)/block_dim_y); 
	CudaI420_To_RGBA<<<blocks, threads>>>(cuda_rgba_buffer, cuda_i420_buffer_Y, cuda_i420_buffer_UV, width, height);
}

